Module: Ecoportal::API::Common::Concerns::Benchmarkable

Defined in:
lib/ecoportal/api/common/concerns/benchmarkable.rb

Overview

Note:

While benchmarking with multi-thread gives correct results for top unique block, will calculate the real time of each individual thread, which is way higher than a single thread. However, as they run in parallel, the total time would be the valid reference.